namespace CCUpdate;

interface

uses System.Collections.Generic;

type
  CortexCommand = public class
  protected
    class property CortexDir: String;
    class method GetDir: String;
  public
    class property CortexDirectory: String read GetDir;
    class method UpdateModule(ModuleName: String; FindList, ReplaceList: Array of String): Array[0..1] of Object;
    class method GetModules: Array of String;
  end;
  
implementation

class method CortexCommand.GetDir: String; 
begin
  if Assigned(CortexDir) then
    exit CortexDir;
  
  var key: Microsoft.Win32.RegistryKey := Microsoft.Win32.Registry.LocalMachine.OpenSubKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall');
  for each subkey: String in key.GetSubKeyNames() do
    using tempkey: Microsoft.Win32.RegistryKey := key.OpenSubKey(subkey) do
      if (tempkey.GetValue('DisplayName', 'None') as String).Contains('Cortex Command') then begin
        CortexDir := tempkey.GetValue('InstallLocation').ToString().TrimEnd(['/','\']);
        Result := CortexDir;
        break;
      end;
end;

class method CortexCommand.UpdateModule(ModuleName: String; FindList, ReplaceList: Array of String): Array[0..1] of Object;
begin
  var stack: Stack<String> := new Stack<String>;
  var ModuleDir: String := CortexCommand.CortexDirectory + '\' + ModuleName.Trim(['/','\']);
  if not System.IO.Directory.Exists(ModuleDir) then
    exit [false, String.Format('The module {0} does not exist.', ModuleName)];
  if FindList.Length <> ReplaceList.Length then
    exit [false, String.Format('The find list and replace list lengths are inequal.  {0} versus {1}.', FindList.Length, ReplaceList.Length)];

  var tempdate: DateTime := DateTime.Now;
  for each file: String in System.IO.Directory.GetFiles(ModuleDir, '*.ini', System.IO.SearchOption.AllDirectories) do
    if not file.ToLower.Contains('index.ini') then begin
      stack.Push(System.IO.File.ReadAllText(file, System.Text.Encoding.ASCII));
      for i: Int32 := 0 to FindList.Length-1 do
        stack.Push(stack.Pop.Replace(FindList[i], ReplaceList[i]));
      System.IO.File.WriteAllText(file, stack.Pop);
    end;
  exit [true, String.Format('Module {0} successfully updated in {1,4}ms.',[ModuleName, TimeSpan(DateTime.Now-tempdate).TotalMilliseconds])];
end;

class method CortexCommand.GetModules: Array of String; 
begin
  Result := System.IO.Directory.GetDirectories(GetDir, '*.rte');
end;

end.